@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Free Mandatory Access Control.
  *
- * Copyright (C) 2005-2008  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.6.0   2008/04/01
+ * Version: 1.6.9+   2012/05/05
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -24,28 +24,39 @@
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 #include <linux/namei.h>
 #endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
+#include <linux/nsproxy.h>
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+#define PATH_or_NAMEIDATA path
+#else
+#define PATH_or_NAMEIDATA nameidata
+#endif
 
 /**
- * check_conceal_mount - Check whether this mount request shadows existing mounts.
+ * ccs_check_conceal_mount - Check whether this mount request shadows existing mounts.
  *
- * @nd:     Pointer to "struct nameidata".
+ * @path:   Pointer to "struct path" (for 2.6.27 and later).
+ *          Pointer to "struct nameidata" (for 2.6.26 and earlier).
  * @vfsmnt: Pointer to "struct vfsmount".
  * @dentry: Pointer to "struct dentry".
  *
  * Returns true if @vfsmnt is parent directory compared to @nd, false otherwise.
  */
-static bool check_conceal_mount(struct nameidata *nd, struct vfsmount *vfsmnt,
-				struct dentry *dentry)
+static bool ccs_check_conceal_mount(struct PATH_or_NAMEIDATA *path,
+				    struct vfsmount *vfsmnt,
+				    struct dentry *dentry)
 {
 	/***** CRITICAL SECTION START *****/
 	while (1) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-		if (nd->path.mnt->mnt_root == vfsmnt->mnt_root &&
-		    nd->path.dentry == dentry)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25) && LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 26)
+		if (path->path.mnt->mnt_root == vfsmnt->mnt_root &&
+		    path->path.dentry == dentry)
 			return true;
 #else
-		if (nd->mnt->mnt_root == vfsmnt->mnt_root &&
-		    nd->dentry == dentry)
+		if (path->mnt->mnt_root == vfsmnt->mnt_root &&
+		    path->dentry == dentry)
 			return true;
 #endif
 		if (dentry == vfsmnt->mnt_root || IS_ROOT(dentry)) {
@@ -62,30 +73,32 @@ static bool check_conceal_mount(struct nameidata *nd, struct vfsmount *vfsmnt,
 }
 
 /**
- * print_error - Print error message.
+ * ccs_print_error - Print error message.
  *
- * @nd:   Pointer to "struct nameidata".
- * @mode: Access control mode.
+ * @r:    Pointer to "struct ccs_request_info".
+ * @path: Pointer to "struct path" (for 2.6.27 and later).
+ *        Pointer to "struct nameidata" (for 2.6.26 and earlier).
  *
- * Returns 0 if @mode is not enforcing or permitted by the administrator's
+ * Returns 0 if @r->mode is not enforcing or permitted by the administrator's
  * decision, negative value otherwise.
  */
-static int print_error(struct nameidata *nd, const u8 mode)
+static int ccs_print_error(struct ccs_request_info *r,
+			   struct PATH_or_NAMEIDATA *path)
 {
 	int error;
-	const bool is_enforce = (mode == 3);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-	const char *dir = ccs_realpath_from_dentry(nd->path.dentry,
-						   nd->path.mnt);
+	const bool is_enforce = (r->mode == 3);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25) && LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 26)
+	const char *dir = ccs_realpath_from_dentry(path->path.dentry,
+						   path->path.mnt);
 #else
-	const char *dir = ccs_realpath_from_dentry(nd->dentry, nd->mnt);
+	const char *dir = ccs_realpath_from_dentry(path->dentry, path->mnt);
 #endif
 	const char *exename = ccs_get_exe();
 	printk(KERN_WARNING "SAKURA-%s: mount %s (pid=%d:exe=%s): "
 	       "Permission denied.\n", ccs_get_msg(is_enforce), dir,
-	       current->pid, exename);
+	       (pid_t) sys_getpid(), exename);
 	if (is_enforce)
-		error = ccs_check_supervisor("# %s is requesting\n"
+		error = ccs_check_supervisor(r, "# %s is requesting\n"
 					     "mount on %s\n", exename, dir);
 	else
 		error = 0;
@@ -97,15 +110,15 @@ static int print_error(struct nameidata *nd, const u8 mode)
 /**
  * ccs_may_mount - Check whether this mount request shadows existing mounts.
  *
- * @nd: Pointer to "struct nameidata".
+ * @path: Pointer to "struct nameidata" or "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-int ccs_may_mount(struct nameidata *nd)
+int ccs_may_mount(struct PATH_or_NAMEIDATA *path)
 {
+	struct ccs_request_info r;
 	struct list_head *p;
 	bool flag = false;
-	const u8 mode = ccs_check_flags(CCS_SAKURA_DENY_CONCEAL_MOUNT);
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 	struct namespace *namespace = current->namespace;
 #elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
@@ -113,30 +126,32 @@ int ccs_may_mount(struct nameidata *nd)
 #else
 	struct mnt_namespace *namespace = current->nsproxy->mnt_ns;
 #endif
-	if (!mode)
+	if (!ccs_can_sleep())
+		return 0;
+	ccs_init_request_info(&r, NULL, CCS_DENY_CONCEAL_MOUNT);
+	if (!r.mode)
 		return 0;
 	if (!namespace)
 		return 0;
+ retry:
 	list_for_each(p, &namespace->list) {
 		struct vfsmount *vfsmnt = list_entry(p, struct vfsmount,
 						     mnt_list);
 		struct dentry *dentry = vfsmnt->mnt_root;
 		/***** CRITICAL SECTION START *****/
-		spin_lock(&dcache_lock);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
-		spin_lock(&vfsmount_lock);
-#endif
+		ccs_realpath_lock();
 		if (IS_ROOT(dentry) || !d_unhashed(dentry))
-			flag = check_conceal_mount(nd, vfsmnt, dentry);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
-		spin_unlock(&vfsmount_lock);
-#endif
-		spin_unlock(&dcache_lock);
+			flag = ccs_check_conceal_mount(path, vfsmnt, dentry);
+		ccs_realpath_unlock();
 		/***** CRITICAL SECTION END *****/
 		if (flag)
 			break;
 	}
-	if (flag)
-		return print_error(nd, mode);
+	if (flag) {
+		int error = ccs_print_error(&r, path);
+		if (error == 1)
+			goto retry;
+		return error;
+	}
 	return 0;
 }
