@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Free Mandatory Access Control.
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.6.7+   2009/04/10
+ * Version: 1.6.9+   2012/05/05
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -37,56 +37,6 @@ static inline void module_put(struct module *module)
 }
 #endif
 
-/**
- * ccs_encode: Encode binary string to ascii string.
- *
- * @str: String in binary format.
- *
- * Returns pointer to @str in ascii format on success, NULL otherwise.
- *
- * This function uses ccs_alloc(), so caller must ccs_free() if this function
- * didn't return NULL.
- */
-static char *ccs_encode(const char *str)
-{
-	int len = 0;
-	const char *p = str;
-	char *cp;
-	char *cp0;
-	if (!p)
-		return NULL;
-	while (*p) {
-		const unsigned char c = *p++;
-		if (c == '\\')
-			len += 2;
-		else if (c > ' ' && c < 127)
-			len++;
-		else
-			len += 4;
-	}
-	len++;
-	cp = ccs_alloc(len, false);
-	if (!cp)
-		return NULL;
-	cp0 = cp;
-	p = str;
-	while (*p) {
-		const unsigned char c = *p++;
-		if (c == '\\') {
-			*cp++ = '\\';
-			*cp++ = '\\';
-		} else if (c > ' ' && c < 127) {
-			*cp++ = c;
-		} else {
-			*cp++ = '\\';
-			*cp++ = (c >> 6) + '0';
-			*cp++ = ((c >> 3) & 7) + '0';
-			*cp++ = (c & 7) + '0';
-		}
-	}
-	return cp0;
-}
-
 /* Keywords for mount restrictions. */
 
 /* Allow to call 'mount --bind /source_dir /dest_dir' */
@@ -154,7 +104,6 @@ static int ccs_update_mount_acl(const char *dev_name, const char *dir_name,
 	if (!fs)
 		return -EINVAL;
 	if (!dev_name)
-		/* Map dev_name to "<NULL>" for if no dev_name given. */
 		dev_name = "<NULL>";
 	if (!strcmp(fs->name, MOUNT_REMOUNT_KEYWORD))
 		/* Fix dev_name to "any" for remount permission. */
@@ -220,7 +169,9 @@ static int ccs_update_mount_acl(const char *dev_name, const char *dir_name,
 		printk(KERN_CONT "%sAllow mount %s %s with options 0x%lX.\n",
 		       ccs_log_level, fs->name, dir->name, ptr->flags);
 	} else {
+		mutex_unlock(&lock);
 		type = get_fs_type(fs->name);
+		mutex_lock(&lock);
 		if (type && (type->fs_flags & FS_REQUIRES_DEV) != 0)
 			printk(KERN_CONT "%sAllow mount -t %s %s %s "
 			       "with options 0x%lX.\n", ccs_log_level,
@@ -368,71 +319,41 @@ static int ccs_check_mount_permission2(struct ccs_request_info *r,
 	int error;
  retry:
 	error = -EPERM;
-	if (!type)
-		type = "<NULL>";
-	if (!dev_name)
-		dev_name = "<NULL>";
 	if ((flags & MS_MGC_MSK) == MS_MGC_VAL)
 		flags &= ~MS_MGC_MSK;
-	switch (flags & (MS_REMOUNT | MS_MOVE | MS_BIND)) {
-	case MS_REMOUNT:
-	case MS_MOVE:
-	case MS_BIND:
-	case 0:
-		break;
-	default:
-		printk(KERN_WARNING "SAKURA-ERROR: "
-		       "%s%s%sare given for single mount operation.\n",
-		       flags & MS_REMOUNT ? "'remount' " : "",
-		       flags & MS_MOVE    ? "'move' " : "",
-		       flags & MS_BIND    ? "'bind' " : "");
-		return -EINVAL;
-	}
-	switch (flags & (MS_UNBINDABLE | MS_PRIVATE | MS_SLAVE | MS_SHARED)) {
-	case MS_UNBINDABLE:
-	case MS_PRIVATE:
-	case MS_SLAVE:
-	case MS_SHARED:
-	case 0:
-		break;
-	default:
-		printk(KERN_WARNING "SAKURA-ERROR: "
-		       "%s%s%s%sare given for single mount operation.\n",
-		       flags & MS_UNBINDABLE ? "'unbindable' " : "",
-		       flags & MS_PRIVATE    ? "'private' " : "",
-		       flags & MS_SLAVE      ? "'slave' " : "",
-		       flags & MS_SHARED     ? "'shared' " : "");
-		return -EINVAL;
-	}
 	if (flags & MS_REMOUNT) {
-		error = ccs_check_mount_permission2(r, dev_name, dir_name,
-						    MOUNT_REMOUNT_KEYWORD,
-						    flags & ~MS_REMOUNT);
-	} else if (flags & MS_MOVE) {
-		error = ccs_check_mount_permission2(r, dev_name, dir_name,
-						    MOUNT_MOVE_KEYWORD,
-						    flags & ~MS_MOVE);
+		type = MOUNT_REMOUNT_KEYWORD;
+		flags &= ~MS_REMOUNT;
 	} else if (flags & MS_BIND) {
-		error = ccs_check_mount_permission2(r, dev_name, dir_name,
-						    MOUNT_BIND_KEYWORD,
-						    flags & ~MS_BIND);
-	} else if (flags & MS_UNBINDABLE) {
-		error = ccs_check_mount_permission2(r, dev_name, dir_name,
-					    MOUNT_MAKE_UNBINDABLE_KEYWORD,
-						    flags & ~MS_UNBINDABLE);
-	} else if (flags & MS_PRIVATE) {
-		error = ccs_check_mount_permission2(r, dev_name, dir_name,
-						    MOUNT_MAKE_PRIVATE_KEYWORD,
-						    flags & ~MS_PRIVATE);
-	} else if (flags & MS_SLAVE) {
-		error = ccs_check_mount_permission2(r, dev_name, dir_name,
-						    MOUNT_MAKE_SLAVE_KEYWORD,
-						    flags & ~MS_SLAVE);
+		type = MOUNT_BIND_KEYWORD;
+		flags &= ~MS_BIND;
 	} else if (flags & MS_SHARED) {
-		error = ccs_check_mount_permission2(r, dev_name, dir_name,
-						    MOUNT_MAKE_SHARED_KEYWORD,
-						    flags & ~MS_SHARED);
-	} else {
+		if (flags & (MS_PRIVATE | MS_SLAVE | MS_UNBINDABLE))
+			return -EINVAL;
+		type = MOUNT_MAKE_SHARED_KEYWORD;
+		flags &= ~MS_SHARED;
+	} else if (flags & MS_PRIVATE) {
+		if (flags & (MS_SHARED | MS_SLAVE | MS_UNBINDABLE))
+			return -EINVAL;
+		type = MOUNT_MAKE_PRIVATE_KEYWORD;
+		flags &= ~MS_PRIVATE;
+	} else if (flags & MS_SLAVE) {
+		if (flags & (MS_SHARED | MS_PRIVATE | MS_UNBINDABLE))
+			return -EINVAL;
+		type = MOUNT_MAKE_SLAVE_KEYWORD;
+		flags &= ~MS_SLAVE;
+	} else if (flags & MS_UNBINDABLE) {
+		if (flags & (MS_SHARED | MS_PRIVATE | MS_SLAVE))
+			return -EINVAL;
+		type = MOUNT_MAKE_UNBINDABLE_KEYWORD;
+		flags &= ~MS_UNBINDABLE;
+	} else if (flags & MS_MOVE) {
+		type = MOUNT_MOVE_KEYWORD;
+		flags &= ~MS_MOVE;
+	}
+	if (!type)
+		type = "<NULL>";
+	{
 		struct ccs_mount_entry *ptr;
 		struct file_system_type *fstype = NULL;
 		const char *requested_type = NULL;
@@ -441,6 +362,7 @@ static int ccs_check_mount_permission2(struct ccs_request_info *r,
 		struct ccs_path_info rdev;
 		struct ccs_path_info rdir;
 		int need_dev = 0;
+
 		requested_type = ccs_encode(type);
 		if (!requested_type) {
 			error = -ENOMEM;
@@ -482,6 +404,9 @@ static int ccs_check_mount_permission2(struct ccs_request_info *r,
 				goto cleanup;
 			}
 		} else {
+			/* Map dev_name to "<NULL>" if no dev_name given. */
+			if (!dev_name)
+				dev_name = "<NULL>";
 			requested_dev_name = ccs_encode(dev_name);
 			if (!requested_dev_name) {
 				error = -ENOMEM;
