@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Based Mandatory Access Control.
  *
- * Copyright (C) 2005-2011  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.6.9   2011/04/01
+ * Version: 1.6.9+   2012/05/05
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -367,11 +367,11 @@ int ccs_read_grant_log(struct ccs_io_buffer *head)
  * ccs_poll_grant_log - Wait for a grant log.
  *
  * @file: Pointer to "struct file".
- * @wait: Pointer to "poll_table".
+ * @wait: Pointer to "poll_table". May be NULL.
  *
- * Returns POLLIN | POLLRDNORM when ready to read a grant log.
+ * Returns POLLIN | POLLRDNORM if ready to read a grant log, 0 otherwise.
  */
-int ccs_poll_grant_log(struct file *file, poll_table *wait)
+unsigned int ccs_poll_grant_log(struct file *file, poll_table *wait)
 {
 	if (ccs_grant_log_count)
 		return POLLIN | POLLRDNORM;
@@ -421,11 +421,11 @@ int ccs_read_reject_log(struct ccs_io_buffer *head)
  * ccs_poll_reject_log - Wait for a reject log.
  *
  * @file: Pointer to "struct file".
- * @wait: Pointer to "poll_table".
+ * @wait: Pointer to "poll_table". May be NULL.
  *
- * Returns POLLIN | POLLRDNORM when ready to read a reject log.
+ * Returns POLLIN | POLLRDNORM if ready to read a reject log, 0 otherwise.
  */
-int ccs_poll_reject_log(struct file *file, poll_table *wait)
+unsigned int ccs_poll_reject_log(struct file *file, poll_table *wait)
 {
 	if (ccs_reject_log_count)
 		return POLLIN | POLLRDNORM;
