@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Based Mandatory Access Control.
  *
- * Copyright (C) 2005-2011  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.6.9   2011/04/01
+ * Version: 1.6.9+   2012/05/05
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -1209,7 +1209,7 @@ static inline struct ipv6hdr *ipv6_hdr(const struct sk_buff *skb)
 #endif
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 12)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
 static void skb_kill_datagram(struct sock *sk, struct sk_buff *skb,
 			      unsigned int flags)
 {
@@ -1306,11 +1306,17 @@ int ccs_socket_recvmsg_permission(struct sock *sk, struct sk_buff *skb,
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
 	if (type == SOCK_DGRAM)
 		lock_sock(sk);
+#elif defined(RHEL_MAJOR) && RHEL_MAJOR == 5 && defined(RHEL_MINOR) && RHEL_MINOR >= 2
+	if (type == SOCK_DGRAM && family != PF_UNIX)
+		lock_sock(sk);
 #endif
 	skb_kill_datagram(sk, skb, flags);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
 	if (type == SOCK_DGRAM)
 		release_sock(sk);
+#elif defined(RHEL_MAJOR) && RHEL_MAJOR == 5 && defined(RHEL_MINOR) && RHEL_MINOR >= 2
+	if (type == SOCK_DGRAM && family != PF_UNIX)
+		release_sock(sk);
 #endif
 	/* Hope less harmful than -EPERM. */
 	return -ENOMEM;
